"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const fs = __importStar(require("fs"));
const readline = __importStar(require("readline"));
const js_yaml_1 = require("js-yaml");
const async = __importStar(require("../async"));
const gatherStats_1 = require("../gatherStats");
const socketMode = {
    windows: "//./pipe/docker_engine",
    unix: "/var/run/docker.sock",
};
exports.run = async (args) => {
    const config = js_yaml_1.safeLoad(fs.readFileSync(args.path, "utf8"));
    fs.lstatSync(config.out_dir).isDirectory();
    const socketPath = socketMode[args.socketMode];
    for (const preCommand of config.pre_commands) {
        const result = await async.exec(preCommand);
        if (result.stderr) {
            console.error(result.stderr);
        }
        else {
            console.log(result.stdout);
        }
    }
    for (let sample = 0; sample <= args.samples; sample++) {
        console.log(`Sample #${sample}`);
        const directory = `${config.out_dir}/${config.project}/${args.revision}/${sample}`;
        await async.mkdir(directory, { recursive: true });
        const stats = [];
        // spawn the processes before the dockers commands are executed
        const childProcesses = config.containers.map((container) => {
            return gatherStats_1.gatherStats(container, socketPath, (statsStringify) => {
                stats.push({ container, stats: statsStringify });
            });
        });
        for (const command of config.commands) {
            try {
                const result = await async.exec(command);
                if (result.stderr) {
                    console.error(result.stderr);
                }
                else {
                    console.log(result.stdout);
                }
            }
            catch (e) {
                console.error(e.stderr);
                process.exit(e.code);
            }
        }
        const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
        // sleep here to be sure that the gatherStats' callback is executed
        // and also for not killing the processes before the stream is ended
        await sleep(5000);
        childProcesses.forEach((childProcess) => {
            childProcess.kill();
        });
        await sleep(1000);
        const intervals = [];
        const file = readline.createInterface({
            input: fs.createReadStream(config.timeline),
            crlfDelay: Infinity,
        });
        for await (const line of file) {
            intervals.push(JSON.parse(line));
        }
        stats.map(({ container, stats }) => {
            const dataJSON = JSON.stringify({
                stats: JSON.parse(stats),
                intervals,
            });
            async.writeFile(`${directory}/${container}.json`, dataJSON);
        });
    }
};
